require 'yaml'
require 'digest/md5'

class Emcee
  class Cache
    @@cache_strategy = nil

    def self.set_strategy(cache_strategy)
      raise "Cache strategy must be one of :memcache, :file, :none, or nil" unless [:memcache, :file, :none, nil].include?(cache_strategy)
      @@cache_strategy = cache_strategy
    end

    def self.strategy
      @@cache_strategy || :memcache
    end

    def self.using_default_strategy?
      @@cache_strategy.nil?
    end

    # factory method to return the appropriate Emcee::Cache strategy instance
    def self.new(*args)
      return memcache_or_filecache_instance(*args) if using_default_strategy?
      case strategy
      when :memcache
        memcache_instance(*args)
      when :file
        filecache_instance(*args)
      when :none
        nullcache_instance(*args)
      else
        raise "Cannot create cache for unknown strategy [#{strategy}]"
      end
    end

    def self.memcache_or_filecache_instance(*args)
      memcache_instance(*args)
    rescue RuntimeError
      begin
        filecache_instance(*args)
      rescue RuntimeError
        raise "Cannot create memcache cache instance nor file cache instance.  Giving up!"
      end
    end

    def self.memcache_instance(*args)
      require 'emcee/memcache_cache'
      Emcee::MemcacheCache.new(*args)  # use memcache caching if it is available
    rescue LoadError
      raise "Cannot create memcache cache instance.  Giving up!"
    end

    def self.filecache_instance(*args)
      require 'emcee/file_cache'
      Emcee::FileCache.new(*args)    # otherwise fall back to file-based caching
    rescue LoadError
      raise "Cannot create file cache instance.  Giving up!"
    end

    def self.nullcache_instance(*args)
      require 'emcee/null_cache'
      Emcee::NullCache.new(*args)
    rescue LoadError
      raise "Cannot create null cache instance.  Giving up!"
    end
  end
end
