require 'spec_helper'
require 'emcee'
require 'emcee/cache'
require 'emcee/memcache_cache'
require 'emcee/file_cache'
require 'emcee/null_cache'

describe Emcee::Cache do
  after :each do
    Emcee::Cache.set_strategy(nil)
  end

  describe 'setting the cache strategy' do
    [:memcache, :file, :none].each do |strategy|
      it "does not fail when #{strategy} is specified" do
        lambda { Emcee::Cache.set_strategy(strategy) }.should_not raise_error
      end
    end

    it 'fails when a strategy other than :memcache, :file, :none, or nil is specified' do
      lambda { Emcee::Cache.set_strategy(:bacon) }.should raise_error
    end
  end

  describe 'determining if the default cache strategy is in use' do
    it 'is true when a nil strategy has been set' do
      Emcee::Cache.set_strategy(nil)
      Emcee::Cache.using_default_strategy?.should be_true
    end

    it 'is false when a non-nil strategy has been set' do
      Emcee::Cache.set_strategy(:file)
      Emcee::Cache.using_default_strategy?.should be_false
    end
  end

  describe 'fetching the current cache strategy' do
    describe 'when a cache strategy has been specified externally' do
      [:memcache, :file, :none].each do |strategy|
        it "returns #{strategy} when :memcache was specified" do
          Emcee::Cache.set_strategy(strategy)
          Emcee::Cache.strategy.should == strategy
        end
      end
    end

    describe 'when a cache strategy has been specified and then cleared via specifying nil' do
      it 'returns :memcache' do
        Emcee::Cache.set_strategy(:file)
        Emcee::Cache.set_strategy(nil)
        Emcee::Cache.strategy.should == :memcache
      end
    end
  end

  describe 'allocating a cache instance' do
    describe 'when no specific cache strategy is specified' do
      before do
        Emcee::Cache.set_strategy(nil)
      end

      it 'returns a memcache caching instance when it is possible to cache via memcache' do
        memcache = stub()
        Emcee::MemcacheCache.stubs(:new).returns(memcache)
        Emcee::Cache.new('key').should == memcache
      end

      it 'returns a file caching instance when it is impossible to cache via memcache but possible to cache via files' do
        memcache = stub()
        filecache = stub()
        Emcee::MemcacheCache.stubs(:new).raises(LoadError)
        Emcee::FileCache.stubs(:new).returns(filecache)
        Emcee::Cache.new('key').should == filecache
      end

      it 'fails when it is not possible to cache via memcache nor files' do
        Emcee::MemcacheCache.stubs(:new).raises(LoadError)
        Emcee::FileCache.stubs(:new).raises(LoadError)
        lambda { Emcee::Cache.new('key') }.should raise_error(RuntimeError)
      end
    end

    describe 'when the :memcache strategy is specified' do
      before :each do
        Emcee::Cache.set_strategy(:memcache)
      end

      it 'returns a memcache caching instance when it is possible to cache via memcache' do
        memcache = stub()
        Emcee::MemcacheCache.stubs(:new).returns(memcache)
        Emcee::Cache.new('key').should == memcache
      end

      it 'fails when it is not possible to cache via memcache' do

        Emcee::MemcacheCache.stubs(:new).raises(LoadError)
        lambda { Emcee::Cache.new('key') }.should raise_error(RuntimeError)
      end
    end

    describe 'when the :file strategy is specified' do
      before :each do
        Emcee::Cache.set_strategy(:file)
      end

      it 'returns a file caching instance when it is possible to cache via file' do
        filecache = stub()
        Emcee::FileCache.stubs(:new).returns(filecache)
        Emcee::Cache.new('key').should == filecache
      end

      it 'fails when it is not possible to cache via file' do
        Emcee::FileCache.stubs(:new).raises(LoadError)
        lambda { Emcee::Cache.new('key') }.should raise_error(RuntimeError)
      end
    end

    describe 'when the :none strategy is specified' do
      before :each do
        Emcee::Cache.set_strategy(:none)
      end

      it 'returns a null caching instance when it is possible to do null caching' do
        nullcache = stub()
        Emcee::NullCache.stubs(:new).returns(nullcache)
        Emcee::Cache.new('key').should == nullcache
      end

      it 'fails when it is not possible to do null caching' do
        Emcee::NullCache.stubs(:new).raises(LoadError)
        lambda { Emcee::Cache.new('key') }.should raise_error(RuntimeError)
      end
    end

    describe 'when an unknown caching strategy is somehow specified' do
      it 'fails' do
        Emcee::Cache.stubs(:using_default_strategy?).returns(false)
        Emcee::Cache.stubs(:strategy).returns(:foo)
        lambda { Emcee::Cache.new('key') }.should raise_error(RuntimeError)
      end
    end
  end
end
