require 'spec_helper'
require 'emcee'
require 'emcee/memcache_cache'

class Emcee::CacheBase
  def log(message)
  end
end

describe Emcee::MemcacheCache do
  before do
    @key = { 'foo' => { 'bar' => 'baz' }, 'xyzzy' => [ 1, 2, 3, [ 'a', 'b', 'c' ] ] }
  end

  describe 'when initializing' do
    it 'accepts a key' do
      lambda { Emcee::MemcacheCache.new(@key) }.should_not raise_error(ArgumentError)
    end

    it 'requires a key' do
      lambda { Emcee::MemcacheCache.new }.should raise_error(ArgumentError)
    end

    it 'makes the key available after initialization' do
      Emcee::MemcacheCache.new(@key).key.should == @key
    end
  end

  describe 'when accessing data' do
    before do
      @cache = Emcee::MemcacheCache.new(@key)
    end

    describe 'and data has not been seen before' do

      it 'runs the data block' do
        lambda { @cache.data do raise "flunk" end }.should raise_error(RuntimeError)
      end

      it 'returns the data' do
        @cache.data do [ 'x', 'y' ] end.should == [ 'x', 'y' ]
      end

    end

    describe 'and data has been seen before' do
      before do
        @cache.data do [ 'x', 'y' ] end
      end

      describe 'and cached data is stale' do
        before do
          @cache.stubs(:cached?).returns(false)
        end

        it 'runs the data block' do
          lambda {
            @cache.data do
              raise "flunk"
            end
          }.should raise_error(RuntimeError)
        end

        it 'returns the data' do
          @cache.data do [ 'x', 'y' ] end.should == [ 'x', 'y' ]
        end
      end

      describe 'and cached data is not stale' do
        before do
          @cache.stubs(:cached?).returns(true)
        end

        it 'does not store the data' do
          @cache.expects(:save).never
          @cache.data do [ 'x', 'y' ] end
        end

        it 'does not run the data block' do
          lambda { @cache.data do raise "flunk" end }.should_not raise_error(RuntimeError)
        end

        it 'returns the cached data' do
          @cache.data do [ 'x', 'y' ] end.should == [ 'x', 'y' ]
        end
      end
    end
  end

  describe 'when manually expiring the cache' do
    before do
      @cache = Emcee::MemcacheCache.new(@key)
    end

    describe 'when a value is set for the key before expiry' do
      before do
        @cache.data do
          'bacon'
        end
      end

      it 'should not have a value cached for the key empty cache after expiring' do
        @cache.expire!
        @cache.cached?.should be_false
      end
    end

    describe 'when no value is set for the key before expiry' do
      it 'should not have a value cached for the key empty cache after expiring' do
        @cache.expire!
        @cache.cached?.should be_false
      end
    end
  end
end
