require 'spec_helper'
require 'emcee/resource'

describe Emcee::Resource do
  before do
    @runner = Emcee::Runner.new
    @emcee = Emcee.new
  end

  describe 'searching for resources of a given type' do
    before do
      @type = 'user'
      @results = YAML.load(File.read(fixture_location('puppetral_user_search_results.yml')))
      @action = stub(:run => @results)
      Emcee::Resource.stubs(:puppetral_action).returns(@action)
    end

    it 'runs the RAL search action' do
      Emcee::Resource.expects(:puppetral_action).with(@emcee, 'search').returns(@action)
      Emcee::Resource.search(@emcee, @type)
    end

    it 'runs the search action for the specified resource type' do
      @action.expects(:run).with({ :type => @type }, nil).returns(@results)
      Emcee::Resource.search(@emcee, @type)
    end

    it 'returns the result of running the RAL search action for the specified resource type' do
      Emcee::Resource.search(@emcee, @type).should == Emcee::Resource.convert_results_list(@results)
    end

    it 'should convert the results of running the RAL search to a consumable format' do
      Emcee::Resource.search(@emcee, @type).should == YAML.load(File.read(fixture_location('converted_search_results.yml')))
    end

    it 'should remove resources marked as absent' do
      output = YAML.load(File.read(fixture_location('converted_search_results.yml')))
      absent = output.first
      output = output[1..-1]
      @results.each do |mc_result|
        if (mc_result.node_name == absent['node_name'] &&
            mc_result.contents.has_key?(absent['title'])
            mc_result.contents[absent['title']]['type'].downcase == absent['type'].downcase)
          mc_result.contents[absent['title']]['parameters'][:ensure] = :absent
          break
        end
      end
      Emcee::Resource.search(@emcee, @type).should == output
    end
  end

  describe 'finding a resource given a type and title' do
    before do
      @type = 'user'
      @title = 'hawkman'
      @results = YAML.load(File.read(fixture_location('puppetral_user_find_results.yml')))
      @action = stub(:run => @results)
      Emcee::Resource.stubs(:puppetral_action).returns(@action)
    end

    it 'runs the RAL find action' do
      Emcee::Resource.expects(:puppetral_action).with(@emcee, 'find').returns(@action)
      Emcee::Resource.find(@emcee, @type, @title)
    end

    it 'runs the RAL find action for the specified resource type and title' do
      @action.expects(:run).with({ :type => @type, :title => @title }, nil).returns(@results)
      Emcee::Resource.find(@emcee, @type, @title)
    end

    it 'returns the converted result of running the RAL find action for the specified resource type and title' do
      Emcee::Resource.find(@emcee, @type, @title).should == Emcee::Resource.convert_results(@results)
    end

    it 'should omit all absent resources by default' do
      result = Emcee::Resource.find(@emcee, @type, @title)
      result.collect { |res| res["node_name"] }.should =~ ['dashnode2', 'dashnode6']
    end

    it 'should omit absent resources except those managed by Puppet when a flag is passed' do
      result = Emcee::Resource.find(@emcee, @type, @title, true)
      result.collect { |res| res["node_name"] }.should =~ ['dashnode2', 'dashnode5', 'dashnode6', 'dashnode7']
    end
  end

  describe 'creating a RAL resource from a resource description' do
    before do
      @result = 'fake result'
      @action = stub(:run => @result)
      Emcee::Resource.stubs(:puppetral_action).returns(@action)
      @resource = { 'type' => 'fakeresource', 'title' => 'faketitle', 'parameters' => {:ensure => :present} }
    end

    it 'runs the RAL create action' do
      Emcee::Resource.expects(:puppetral_action).with(@emcee, 'create').returns(@action)
      Emcee::Resource.create(@emcee, @resource)
    end

    it 'runs the create action for the specified resource type and title' do
      @action.expects(:run).with(@resource, nil).returns(@result)
      Emcee::Resource.create(@emcee, @resource)
    end

    it 'returns the result of running the create action for the specified resource type and title' do
      Emcee::Resource.create(@emcee, @resource).should == @result
    end
  end

  describe 'formatting list of Emcee::Result user data for public consumption' do
    before do
      @yaml_results = File.read(fixture_location('puppetral_user_find_results.yml'))
      @results = YAML.load(@yaml_results)
      undesirables = [:absent, :purged]
      @present_results = @results.reject { |result| undesirables.include?(result.contents['parameters'][:ensure]) }
    end

    it 'should return the empty list when no Emcee::Results are supplied' do
      Emcee::Resource.convert_results([]).should == []
    end

    it 'should return a list of hashes when Emcee::Results are supplied' do
      converted = Emcee::Resource.convert_results(@results)
      converted.each {|result| result.class.should == Hash }
    end

    # [
    #   { :node_name => 'foo', :type => 'user', :title => 'root', :managed => 'true', :parameters => { :ensure => 'present', * } }, # ...
    # ]

    it 'should record the node name for the result as the :node_name hash value' do
      converted = Emcee::Resource.convert_results(@results)
      @present_results.each_with_index do |result, i|
        converted[i]['node_name'].should == result.node_name
      end
    end

    # note: presuming here that conversion process returns a list in the same order as it was provided

    it 'should record the type of the resource as the :type hash value' do
      converted = Emcee::Resource.convert_results(@results)
      @present_results.each_with_index do |result, i|
        converted[i]['type'].should == result.contents['type']
      end
    end

    it 'should record the title of the resource as the :title hash value' do
      converted = Emcee::Resource.convert_results(@results)
      @present_results.each_with_index do |result, i|
        converted[i]['title'].should == result.contents['title']
      end
    end

    it 'should record the puppet management status of the resource as the :managed hash value' do
      converted = Emcee::Resource.convert_results(@results)
      @present_results.each_with_index do |result, i|
        converted[i]['managed'].should == result.contents[:managed]
      end
    end

    it 'should record the parameters hash for the resource as the :parameters hash value' do
      converted = Emcee::Resource.convert_results(@results)
      @present_results.each_with_index do |result, i|
        converted[i]['parameters'].should == result.contents['parameters']
      end
    end

    it 'should not include hashes for any Emcee::Results for users with "ensure" => "absent"' do
      Emcee::Resource.convert_results(@results).each { |result| result['parameters'][:ensure].should_not == :absent }
    end

    it 'should return a hash for every Emcee::Result item supplied with "ensure" => "present"' do
      Emcee::Resource.convert_results(@results).each { |result| result['parameters'][:ensure].should == :present }
    end
  end

  describe "omit_absent_results" do
    it 'should skip and output info about errored results' do
      results = [stub({ :contents => 'error!', :node_name => 'bad_node', :status_message => 'Foo'}),
                 stub({ :contents => {'parameters' => { :ensure => :present }}})]
      $stdout.expects(:write).with("Agent node bad_node had exception while processing request: Foo")
      $stdout.expects(:write).with("\n")
      Emcee::Resource.omit_absent_results(results).should == [results[1]]
    end
  end
end

